/**
 * Location Search Module
 * @param {[type]} id  [description]
 * @param {[type]} url [description]
 */
function LocationSearch(id, url) {
    this.el = $(id);
    var self = this;

    var searchTimer = '';
    this.el.find('.search').on('input propertychange', function(e) {
        var _self = $(this);
        e.preventDefault();
        if (searchTimer) {
            clearTimeout(searchTimer);
        }
        searchTimer = setTimeout(function() {
            var value = _self.val();
            if (value) {
                self.search(Zeus.utils.assembleApi(url + '/' + value));
            }
        }, 500);
    });
}

LocationSearch.prototype.search = function(url) {
    var self = this;
    $.ajax({
        url: url,
        success: function(data) {
            if (data) {
                var htmlTemple = [];
                data.forEach(function(e) {
                    var content = e.city + e.district + e.name;
                    var li = ['<li class="item-content pg-location" data-location=', JSON.stringify(e), '><div class="item-inner">', '<div class="item-title">', content, '</div></li>'];
                    htmlTemple.push(li.join(''));
                });
                self.el.find('.station-list').html(htmlTemple.join(''));
            }
        }
    });
};

LocationSearch.prototype.getLocation = function(target) {
    var displayValue = target.find('.item-title').html();
    var locationValue = JSON.parse(target.data('location'));
    return {
        display: displayValue,
        location: locationValue
    };
};

/**
 * 
 */
$.fn.addition = function(edit) {
    var self = $(this);

    self.status = false;

    self.click(function() {
        if (edit) {
            $(this).trigger('item:edit');
        } else {
            $(this).trigger('item:check');
        }
        $(this).trigger('item:retrun');
    });

    self.bind('item:check', function(e) {

        if (self.status) {
            self.removeClass('checked');
        } else {
            self.addClass('checked');
        }
        self.status = !self.status;

    });

    self.bind('item:edit', function() {
        if (self.status) {
            self.removeClass('checked');
            self.find('.cash').html('(免费)');
        } else {
            self.editBox();
        }
        self.status = !self.status;
    });

    self.editBox = function() {

        var _title = '<div class="rw-title"><div class="con">带回金额</div><div class="line"></div></div>';
        var _text = [];
        _text.push('<div class="pg-input"><span class="tip">金额</span><input type="tel" placeholder="请输入需带回的金额" maxlength="5"/><div class="clear"></div></div>');


        _text = _text.join(' ');
        var _extraClass = 'edit-box';

        var _buttons = [{
            "text": '<strong class="cancel">取消</strong>',
            "onClick": function() {
                self.status = false;
            }
        }, {
            "text": '<strong class="certain">完成</strong>',
            "onClick": function() {
                var cash = $('.' + _extraClass).find('input').val();
                cash = parseInt(cash);
                if (!cash) {
                    $.alert('请输入整数');
                    self.status = false;
                    return;
                }
                if (cash > 10000) {
                    $.alert('金额不能大于10000');
                    self.status = false;
                    return;
                }
                self.status = true;
                self.addClass('checked');
                self.find('.cash').html(cash + '元');
            }
        }];

        var _modal = {
            title: _title,
            text: _text,
            extraClass: _extraClass,
            buttons: _buttons
        };

        $.modal(_modal);
        $('.edit-box .pg-input input')[0].focus();
        $(document).on('input change', '.edit-box .pg-input input', function() {
            if ($(this).val()) {
                $('.edit-box').find('div.clear').show();
            } else {
                $('.edit-box').find('div.clear').hide();
            }
        });

        $(document).on('click', '.edit-box div.clear', function() {
            $('.edit-box input').val('');
            $('.edit-box input')[0].focus();
            $(this).hide();
        });
    };
    return self;
};

$(function() {
    if (!Zeus.utils.Account.get()) {
        return;
    }
    // if device android weixin
    if ($.device.android && $.device.isWeixin) {
        $('input[readonly=readonly]').attr({
            "disabled": 'disabled',
            "readonly": ' '
        });
        $('#pg-date-picker').removeAttr('disabled');
    }

    // post parameter
    var postPara = {
        "from": '',
        "price": '',
        "to": '',
        "vType": {
            "id": 1,
            "carName": "小型面包车"
        },
        "services": [],
        "comment": '',
        "time": '',
        "amountCashed": ''
    };

    // all input select
    var $from = $('#pg-demand-publish input[name=strloc]'),
        $to = $('#pg-demand-publish input[name=endloc]'),
        $time = $('#pg-demand-publish input[name=time]'),
        $comment = $('#pg-demand-publish .comment');

    var valiFlags = {
        from: true,
        to: false,
    };

    var city = '深圳';
    var searchPage = new LocationSearch('#pg-location-search', '/citys/' + city + '/places');

    // validate value
    function flagVali() {
        var flag = valiFlags.from && valiFlags.to;
        if (flag) {
            $('#pg-demand-publish .pg-btn-publish').removeClass('disable');
            return true;
        } else {
            return false;
        }
    }

    // 空值验证
    function nullValidate(value) {
        if (value) {
            return true;
        }
        return false;
    }

    // get price and add 
    function setPrice(vehiceTypeid, from, to) {
        $.ajax({
            url: Zeus.utils.assembleApi('/routes'),
            data: { "vehiceTypeid": vehiceTypeid, "from": from.lat + ',' + from.lng, "to": to.lat + ',' + to.lng },
            success: function(data) {
                postPara.distanceValue = data.distanceValue;
                postPara.timeValue = data.timeValue;
                postPara.price = data.sq.price;
                $('.pg-price-info .factor .route .distance').html(data.distance);
                $('.pg-price-info .factor .route .time').html(parseInt(data.timeValue * 60) + '分钟');
                $('.pg-price-info .price .price-num').html(data.sq.price);
                $('.pg-price-info').show();
            }
        });
    }



    // 一些数据在点击发布前进行赋值
    function setPostPara() {
        postPara.comment = $comment.val();
        postPara.services = [];
        $('.pg-service-info li').each(function(index, el) {
            var self = $(el);
            if (self.hasClass('checked')) {
                postPara.services.push(self.data('service'));
                var cash = self.find('.cash').html();
                postPara.amountCashed = parseInt(cash) || 0;
            }
        });
    }

    // select car
    function selectCar(self, id, carName) {
        self.addClass('active');
        self.siblings('li').removeClass('active');
        self.find('.car .icon-block').addClass('active');
        self.siblings('li').find('.car .icon-block').removeClass('active');

        // set postPara.carType
        postPara.vType.id = id || self.data('cartype') || 1;
        postPara.vType.carName = carName || self.find('.car-name').html() || '小型面包车';

        if (flagVali()) {
            setPrice(postPara.vType.id, postPara.from.location, postPara.to.location);
        }
    }


    var orderData = Zeus.utils.getUrlPara('orderData');
    if (!orderData) {
        // 获取车辆类型
        $.showIndicator();
        $.ajax({
            url: Zeus.utils.assembleApi('/vehicleTypes'),
            success: function(data) {
                $.hideIndicator();
                data.forEach(function(e, i) {
                    $('.pg-cars-info li').eq(i).data("cartype", e.id).children('.car-name').html(e.carName);
                });
            },
            error: function(xhr) {
                Zeus.utils.xhrAlert(xhr);
            }
        });

        // 设置附加服务
        $.ajax({
            url: Zeus.utils.assembleApi('/orders/charge'),
            success: function(data) {
                var services = data.selectServices;
                $('.pg-service-info li').eq(0).addition().data('service', services[0]);
                $('.pg-service-info li').eq(1).addition().data('service', services[1]);
                $('.pg-service-info li').eq(2).addition(true).data('service', services[2]);
            }
        });


        // 获取城市
        Zeus.utils.getCity(function(data) {
            city = data || '深圳';
            searchPage = new LocationSearch('#pg-location-search', '/citys/' + city + '/places');
        }, BMap);

        // 获取默认地址
        Zeus.utils.getGeolocation(function(data) {
            var stValue = $('input[name=strloc]').val();
            if (stValue) {
                return;
            }
            $('input[name=strloc]').val(data.display);
            postPara.from = data.para;
        }, BMap);
    }


    $from.on('change', function(event) {
        var value = $(this).val();
        valiFlags.from = nullValidate(value);
        if (flagVali()) {
            setPrice(postPara.vType.id, postPara.from.location, postPara.to.location);
        }
    });

    $to.on('change', function(event) {
        var value = $(this).val();
        valiFlags.to = nullValidate(value);
        if (flagVali()) {
            setPrice(postPara.vType.id, postPara.from.location, postPara.to.location);
        }
    });

    // init date picker
    $('#pg-date-picker').yunDatePicker({
        toolbarTemplate: '<header class="bar bar-nav">' +
            '<button class="now button button-link pull-left">现在</button>' +
            '<button class="button button-link pull-right close-picker">确定</button>' +
            '<h1 class="title">请选择时间</h1>' +
            '</header>',
        cssClass: 'date-time-picker',
        onClose: function() {
            var time = $('#pg-date-picker').val().split(' ');
            time = time[0] + ' ' + Zeus.utils.formatNum(time[1]) + ':' + Zeus.utils.formatNum(time[2]) + ':00';
            postPara.time = time;
        }
    });


    $(document).on('click', '.picker-modal.date-time-picker .now', function() {
        delete postPara.time;
        $('#pg-demand-publish .pg-time-info .show-time').html('现在');
        $('#pg-date-picker').picker('close');
    });

    // open date picker
    $(document).on('click', '.pg-time-info', function() {
        $('#pg-date-picker').picker('open');
    });

    // select car
    $(document).on('click', '#pg-demand-publish .pg-cars-info li', function() {
        selectCar($(this), $(this).data('cartype'), $(this).find('.car-name').html());
    });


    // goto search page
    var whichStation = '';
    $(document).on('click', '#pg-demand-publish input[name=strloc],input[name=endloc]', function() {
        whichStation = $(this).attr('name');
        $.router.load('#pg-location-search');
    });

    //  location value
    $(document).on('click', '#pg-location-search .pg-location', function() {
        var result = searchPage.getLocation($(this));

        if (whichStation == 'strloc') {
            $from.val(result.display);
            postPara.from = result.location;
            $from.trigger('change');
        }
        if (whichStation == 'endloc') {
            $to.val(result.display);
            postPara.to = result.location;
            $to.trigger('change');
        }
        $.router.load('#pg-demand-publish');
    });

    // before page animation event 
    // clear search input value
    $(document).on('pageAnimationStart', '#pg-location-search', function() {
        $('#pg-location-search input[type=search]').val('');
        $('#pg-location-search .station-list').html('');
        $('#pg-location-search .search').trigger('focus');
    });

    // 点击发布
    $(document).on('click', '#pg-demand-publish .pg-btn-publish', function() {
        if ($(this).hasClass('disable')) {
            return;
        }
        setPostPara();
        $.confirm('确定发布订单?', function() {
            var d = Zeus.utils.parseDate(postPara.time);
            if (d && d.getTime() <= Date.now()) delete postPara.time;

            $.showIndicator();
            $.ajax({
                url: Zeus.utils.assembleApi('/users/me/orders'),
                type: 'POST',
                contentType: 'application/json',
                data: JSON.stringify(postPara),
                success: function(data) {
                    $.hideIndicator();
                    document.location.href = "orderWait.html?orderId=" + data + '#onback-' + encodeURIComponent('orderList.html');
                },
                error: function(xhr, type, error) {
                    $.hideIndicator();
                    Zeus.utils.xhrAlert(xhr);
                }
            });
        });
    });

});
